from sqlalchemy import *
from migrate import *
import migrate.changeset
import re

# discountsystem1 = Table('discountsystem1', meta, autoload=True)
# discountsystem2 = Table('discountsystem2', meta, autoload=True)

class Dictionaries(object):
    def __init__(self, meta):
        self.agent = Table('agent', meta,
                      Column('code', String(100), nullable=False, primary_key=True),
                      Column('name', String(255), nullable=True),
                      Column('inn', String(100), nullable=True),
                      Column('phone', String(100), nullable=True),
                      Column('address', String(100), nullable=True))
    
        self.terminal = Table('terminal', meta,
                         Column('terminalid', Integer, nullable=False, autoincrement=True, primary_key=True),
                         Column('agentcode', String(100), nullable=False),
    #                     Column('agentcode', String(100), ForeignKey('agent.code', name='terminalagentcodefk'), nullable=False),
                         Column('terminalcode', String(100)),
                         Column('address', String(255)))
    
        self.region = Table('region', meta,
                       Column('regionid', Integer, nullable=False, autoincrement=True, primary_key=True),
                       Column('regioncode', String(100), nullable=False),
                       Column('agentcode', String(100), nullable=False),
    #                   Column('agentcode', String(100), ForeignKey('agent.code', name='regionagentcodefk'), nullable=False),
                       Column('name', String(255)))
    
        self.category = Table('category', meta,
                         Column('categoryid', Integer, nullable=False, primary_key=True, autoincrement=True),
                         Column('agentcode', String(100), nullable=False),
    #                     Column('agentcode', String(100), ForeignKey('agent.code', name='categoryagentcodefk'), nullable=False),
                         Column('categorycode', String(100), nullable=False),
                         Column('name', String(255)))
    
        self.provider = Table('provider', meta,
                         Column('providerid', Integer, nullable=False, primary_key=True, autoincrement=True),
                         Column('providercode', String(100), nullable=False),
                         Column('terminalid', Integer, nullable=False),
                         Column('categoryid', Integer, nullable=False),
    #                     Column('agentcode', String(100), ForeignKey('agent.code', name='provideragentcodefk'), nullable=False),
    #                     Column('categoryid', Integer, ForeignKey('category.categorycode', name='providercategorycodefk'), nullable=False),
                         Column('name', String(100)),
                         Column('inn', String(100)),
                         Column('phone', String(100)),
                         Column('address', String(255)),
                         Column('accountmask', String(255)),
                         Column('minsum', Numeric(13, 2)),
                         Column('maxsum', Numeric(13, 2)),
                         Column('bcodemask', String(255)),
                         Column('bcodetransform', String(255)),
                         Column('inputmask', String(255)),
                         Column('inputtransform', String(255)),
                         Column('accountdatamask', String(255)),
                         Column('accountdatatransform', String(255)))
    
        self.accountmask = Table('accountmask', meta,
                            Column('maskid', Integer, nullable=False, primary_key=True, autoincrement=True),
                            Column('providerid', Integer, nullable=False),
    #                         Column('providerid', String(100), ForeignKey('provider.providerid', name='providermaskprovideridfk'), nullable=False),
                            Column('mask', String(200), nullable=False),
                            Column('transform', String(200)))
    
        self.defcode = Table('defcode', meta,
                        Column('defcodeid', Integer, nullable=False, primary_key=True, autoincrement=True),
                        Column('providerid', Integer, nullable=False),
    #                    Column('providerid', String(100), ForeignKey('provider.providerid', name='defcodeprovideridfk'), nullable=False),
                        Column('rangemin', String(100), nullable=False),
                        Column('rangemax', String(100), nullable=False),
                        Column('regionid', Integer, nullable=False))
    
        self.extendedparameter = Table('extendedparameter', meta,
                                   Column('extendedparameterid', Integer, nullable=False, primary_key=True, autoincrement=True),
                                   Column('providerid', Integer, nullable=False),
    #                               Column('providerid', String(100), ForeignKey('provider.providerid', name='extendedparameterprovideridfk'), nullable=False),
                                   Column('code', String(200)),
                                   Column('name', String(200)),
                                   Column('mask', String(200)),
                                   Column('required', Integer))
    
        self.clientcard = Table('clientcardmask', meta,
                           Column('clientcardmaskid', Integer, nullable=False, primary_key=True, autoincrement=True),
                           Column('agentcode', String(255), nullable=False),
                           Column('mask', String(255)),
                           Column('transform', String(255)))


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    # Upgrade operations go here. Don't create your own engine; use the engine
    # named 'migrate_engine' imported from migrate.
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in [ '1', '2' ]:
            table.name = basename + suffix
            table.create()
    conn = migrate_engine.connect()
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name[:-1]
        conn.execute('create view %s as select * from %s1' % (basename, basename))

def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    # Operations to reverse the above upgrade go here.
    conn = migrate_engine.connect()
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        conn.execute('drop view %s' % (basename))
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in [ '1', '2' ]:
            table.name = basename + suffix
            table.drop()
