from sqlalchemy import *
from migrate import *
import migrate.changeset
from dictionaries.versions import views_helper
import re


def getCommandTables(meta):
    command1 = Table('command1', meta, autoload=True)
    command2 = Table('command2', meta, autoload=True)
    return [command1, command2]


class Dictionaries(object):
    def __init__(self, meta):
        self.rolecommand = Table('rolecommand', meta,
                                 Column('id', Integer, nullable=False, primary_key=True, autoincrement=True),
                                 Column('rolecode', Integer, nullable=False),
                                 Column('commandcode', Integer, nullable=False),
                                 Column('rule', Integer, nullable=False),
                                 Column('documenttype', Integer, nullable=False, default=0),
                                 )


def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in ['1', '2']:
            table.name = basename + suffix
            table.create()
    conn = migrate_engine.connect()
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name[:-1]
        conn.execute('create view %s as select * from %s' % (basename, basename + '1'))

    for tbl in getCommandTables(meta):
        logmode = Column('logmode', Integer, nullable=False, server_default='0')
        logmode.create(tbl)
    views_helper.update_dic_view(migrate_engine, 'command')


def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    dictionaries = Dictionaries(meta)
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name
        for suffix in ['1', '2']:
            table.name = basename + suffix
            table.drop()
    conn = migrate_engine.connect()
    for table in filter(lambda x: isinstance(x, Table), map(lambda x: getattr(dictionaries, x), dir(dictionaries))):
        basename = table.name[:-1]
        conn.execute('drop view %s' % (basename))

    tblname = views_helper.get_table_name(migrate_engine, 'command')
    for tbl in getCommandTables(meta):
        tbl.c.logmode.drop()
    views_helper.update_dic_view(migrate_engine, 'command', tblname)
