from sqlalchemy import *
from migrate import *
import migrate.changeset

def getClient(meta):
    client = Table('client', meta,
        Column('clientitemid', Integer, nullable=False, primary_key=True, autoincrement=True),
        Column('idclient', Integer, nullable=True),
        Column('name', String(200), nullable=True),
        Column('text', String(200), nullable=True),
        Column('sex', Integer, nullable=True),
        Column('birthday', Date(), nullable=True),
        Column('specialdate1', Date(), nullable=True),
        Column('specialdate2', Date(), nullable=True),
        Column('specialdate3', Date(), nullable=True),
        Column('specialdate1name', String(200), nullable=True),
        Column('specialdate2name', String(200), nullable=True),
        Column('specialdate3name', String(200), nullable=True),
        Column('zipcode', String(200), nullable=True),
        Column('address', String(200), nullable=True),
        Column('email', String(200), nullable=True),
        Column('webpage', String(200), nullable=True),
        Column('phonenumber', String(200), nullable=True),
        Column('inn', String(200), nullable=True),
        Column('document', String(200), nullable=True),
        Column('okpo', String(200), nullable=True),
        Column('okpd', String(200), nullable=True),
        Column('occupation', String(200), nullable=True),
        Column('childrencount', Integer, nullable=True),
        Column('extendedoptions', Text(), nullable=True),
        mysql_engine='InnoDB'
    )
    return client

def getDepartmentTable(meta):
    return Table('department', meta, autoload=True)

def upgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    client = getClient(meta)
    client.create()
    department = getDepartmentTable(meta)
    Column('clientitemid', Integer, nullable=True).create(department)
    Column('documentprefix', String(20), nullable=True).create(department)
    Column('documentnumber', String(100), nullable=True).create(department)
    migrate.changeset.constraint.ForeignKeyConstraint([department.c.clientitemid], [client.c.clientitemid], name = 'departmentclientitemidfk').create()

def downgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    client = getClient(meta)
    department = getDepartmentTable(meta)
    migrate.changeset.constraint.ForeignKeyConstraint([department.c.clientitemid], [client.c.clientitemid], name = 'departmentclientitemidfk').drop()
    if 'documentprefix' in department.c:
        department.c.documentprefix.drop()
    if 'documentnumber' in department.c:
        department.c.documentnumber.drop()
    department.c.clientitemid.drop()
    client.drop()
