# -*- coding: utf-8 -*-
from sqlalchemy import *
from migrate import *
from sqlalchemy.dialects.mysql.pyodbc import MySQLDialect
from documents.versions import dump
from artix.database.info import DBInfo

def dropTrigger(migrate_engine):
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterUpdate`;")


def createTrigger(migrate_engine):
    dbInfo = DBInfo()
   
    trigger_after_update = """
CREATE TRIGGER triggerAfterUpdate AFTER UPDATE ON document
FOR EACH ROW BEGIN    
    set @oldfactor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);
    set @newfactor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    set @oldsumb=0;
    set @oldsumcash=0;    
    set @newsumb=0;    
    set @newsumcash=0;
    set @oldsumnoncash=0;
    set @newsumnoncash=0;
    set @oldcount=0;
    set @newcount=0;
    set @oldsumother=0;
    set @newsumother=0;

    IF old.closed = 1
    THEN
       set @oldsumb=cast(old.sumb as Decimal(13, 2));
       set @oldsumother=cast(old.sumother as Decimal(13, 2));
       set @oldsumcash=@oldfactor * cast(old.sumcash as Decimal(13, 2));
       set @oldsumnoncash=@oldfactor * cast(old.sumnoncash as Decimal(13, 2));
       set @oldcount=1;
    END IF;

    IF new.closed = 1 
    THEN
       set @newsumb=cast(new.sumb as Decimal(13, 2));
       set @newsumother=cast(new.sumother as Decimal(13, 2));
       set @newsumcash=@newfactor * cast(new.sumcash as Decimal(13, 2));
       set @newsumnoncash=@newfactor * cast(new.sumnoncash as Decimal(13, 2));
       set @newcount=1;
    END IF;        

    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, -@oldsumb + @newsumb, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), -@oldsumb + @newsumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - @oldsumcash + @newsumcash,        
        sumrefund = COALESCE(sumrefund, 0) + if(new.doctype in (2,25), -abs(@oldsumb) + abs(@newsumb), 0),
        sumsalecash = COALESCE(sumsalecash, 0) + if(new.doctype = 1, -@oldsumcash + @newsumcash, 0),        
        sumsalenoncash = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, -@oldsumnoncash + @newsumnoncash, 0),        
        sumsaleother = COALESCE(sumsaleother, 0) + if(new.doctype = 1, -@oldsumother + @newsumother, 0),        
        sumgaincash = COALESCE(sumgaincash, 0) + if(new.doctype in (1,2,25), -@oldsumcash + @newsumcash, 0),        
        sumgainnoncash = COALESCE(sumgainnoncash, 0) + if(new.doctype in (1,2,25), -@oldsumnoncash + @newsumnoncash, 0),        
        sumrefundcash = COALESCE(sumrefundcash, 0) + if(new.doctype in (2,25), -abs(@oldsumcash) + abs(@newsumcash), 0),        
        sumrefundnoncash = COALESCE(sumrefundnoncash, 0) + if(new.doctype in (2,25), -abs(@oldsumnoncash) + abs(@newsumnoncash), 0),        
        countsale = COALESCE(countsale, 0) + if(new.doctype = 1, -@oldcount + @newcount, 0),
        countrefund = COALESCE(countrefund, 0) + if(new.doctype in (2, 25), -@oldcount + @newcount, 0)
    WHERE workshiftid = new.workshiftid;
END
"""
    migrate_engine.connect().execute(trigger_after_update)


def createOldTrigger(migrate_engine):
    dbInfo = DBInfo()
   
    trigger_after_update = """
CREATE TRIGGER triggerAfterUpdate AFTER UPDATE ON document
FOR EACH ROW BEGIN    
    set @oldfactor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);
    set @newfactor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    set @oldsumb=0;
    set @oldsumcash=0;    
    set @newsumb=0;    
    set @newsumcash=0;
    set @oldsumnoncash=0;
    set @newsumnoncash=0;
    set @oldcount=0;
    set @newcount=0;
    set @oldsumother=0;
    set @newsumother=0;

    IF old.closed = 1
    THEN
       set @oldsumb=old.sumb;
       set @oldsumother=old.sumother;
       set @oldsumcash=@oldfactor * old.sumcash;
       set @oldsumnoncash=@oldfactor * old.sumnoncash;
       set @oldcount=1;
    END IF;

    IF new.closed = 1 
    THEN
       set @newsumb=new.sumb;
       set @newsumother=new.sumother;
       set @newsumcash=@newfactor * new.sumcash;
       set @newsumnoncash=@newfactor * new.sumnoncash;
       set @newcount=1;
    END IF;        

    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, -@oldsumb + @newsumb, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), -@oldsumb + @newsumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - @oldsumcash + @newsumcash,        
        sumrefund = COALESCE(sumrefund, 0) + if(new.doctype in (2,25), -abs(@oldsumb) + abs(@newsumb), 0),
        sumsalecash = COALESCE(sumsalecash, 0) + if(new.doctype = 1, -@oldsumcash + @newsumcash, 0),        
        sumsalenoncash = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, -@oldsumnoncash + @newsumnoncash, 0),        
        sumsaleother = COALESCE(sumsaleother, 0) + if(new.doctype = 1, -@oldsumother + @newsumother, 0),        
        sumgaincash = COALESCE(sumgaincash, 0) + if(new.doctype in (1,2,25), -@oldsumcash + @newsumcash, 0),        
        sumgainnoncash = COALESCE(sumgainnoncash, 0) + if(new.doctype in (1,2,25), -@oldsumnoncash + @newsumnoncash, 0),        
        sumrefundcash = COALESCE(sumrefundcash, 0) + if(new.doctype in (2,25), -abs(@oldsumcash) + abs(@newsumcash), 0),        
        sumrefundnoncash = COALESCE(sumrefundnoncash, 0) + if(new.doctype in (2,25), -abs(@oldsumnoncash) + abs(@newsumnoncash), 0),        
        countsale = COALESCE(countsale, 0) + if(new.doctype = 1, -@oldcount + @newcount, 0),
        countrefund = COALESCE(countrefund, 0) + if(new.doctype in (2, 25), -@oldcount + @newcount, 0)
    WHERE workshiftid = new.workshiftid;
END
"""
    migrate_engine.connect().execute(trigger_after_update)


def upgrade(migrate_engine):
    dump.store(migrate_engine)
    meta = MetaData(bind=migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTrigger(migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        createTrigger(migrate_engine)


def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTrigger(migrate_engine)
        
    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        createOldTrigger(migrate_engine)
